/******************************************************************************/
/* cmfreport.c     CICS on Open Systems Monitor Data Formatter   Rags Gardner */
/*                                                                            */
/* Rags Int., Inc.                                              December 1999 */
/* (c) RISC Application Growth Solutions International, Incorporated  1999    */
/*             (817) 267-2554     rags@compuserve.com                         */
/*                                                                            */
/* cmfreport reads CICS CMF binary data and formats performance reports       */
/*                                                                            */
/* cmfopts      Parses command line arguments to select report options        */
/* MonTitle     Formats a title line based on selected report fields          */
/* MonCompare   Compares for include/exclude records                          */
/* MonExpand    Formats a detail report line                                  */
/*                                                                            */
/* The FormatMap array contains the report formatting directives for each     */
/* EMP field.  This includes the selection name, output width, and title      */
/* text.                                                                      */
/*                                                                            */
/******************************************************************************/
#ifdef WIN32
   #include <windows.h>                                            /* windows */
#endif
#include <stdio.h>                                       /* Unix standard I/O */
#include <sys/types.h>                                 /* standard data types */
#include <memory.h>                                                 /* memory */
#include <string.h>                                                /* strings */
#include <cics_types.h>                                    /* CICS data types */
#include <cicsmfmt.h>                             /* monitor formatter header */
#include "cmfreport.h"                                    /* cmfreport header */

/*----------------------------------------*/
/* global storage areas                   */
/*----------------------------------------*/
char EyeCatcher[]=
     "(c) RISC Application Growth Solutions International, Incorporated  1996"
     "  Phone (817) 267-2554   email rags@compuserve.com ";
char   ProgName[128];                                    /* this program name */
struct FieldList_t   FieldList[LIST_SZ];    /* list of selected report fields */
struct ExcIncList_t IncludeList[LIST_SZ];    /* criteria for included records */
struct ExcIncList_t ExcludeList[LIST_SZ];    /* criteria for excluded records */
struct SummaryTable_t Table;   /* output table containing accumulated records */
#include "cmformats.h"                /* cmf report field descriptions header */

int    LineLth=80;                               /* characters per print line */
int    ReportWidth=0;                   /* line size based on selected fields */
int    PageLth=60;                                          /* lines per page */
int    LineNo=1;                                        /* current line count */
int    FieldCount=0;                              /* count of selected fields */
int    IncludeCount=0;                             /* count of include fields */
int    ExcludeCount=0;                             /* count of exclude fields */
int    TitleLines=1;                                  /* count of title lines */
int    TimeInSec=FALSE;       /* timers reported in seconds.milliseconds flag */
int    SummaryRF=FALSE;                     /* accumulate summary report flag */
int    SummaryF=TRUE;                          /* print summary counters flag */
int    PrtDetail=TRUE;                             /* print detail lines flag */
int    PrtCount=FALSE;               /* print counters associated with clocks */
int    GMT=FALSE;                         /* show time stamps in local or GMT */
int    MonFields=0;                  /* number of monitor field types defined */

/*----------------------------------------------------------------------------*/
/*                                                                            */
/* main()                                Main Processing                      */
/*                                                                            */
/*----------------------------------------------------------------------------*/
void main(int argc, char *argv[])
{
  CICS_MFMT_Stream_t           *MonDataIn;      /* CMF input data file stream */
  CICS_EMP_TDQ_RecordDetails_t *ptr;                    /* CMF data structure */
  enum CICS_MFMT_Status         status;             /* input file status flag */
  struct FieldDesc_t           *ReportItem;   /* report description structure */
  struct SummaryRow_t           temp_row;    /* work space for creating a row */
  struct SummaryRow_t          *tmpPtr;                    /* a loop iterator */
  char                          MonDataFile[128]="";    /* CMF data file name */
  char                          field[128];         /* report field work area */
  char                          TopTitle[PRT_MAX+1]="";     /* top title text */
  char                          FieldTitle[PRT_MAX+1];  /* heading title text */
  char                          DetailLine[PRT_MAX+1];           /* work area */
  int                           atEOF=FALSE;              /* end of file flag */
  int                           i, i2;                       /* work integers */
  int                           PageNo=1;               /* current page count */
  int                           Select=TRUE;               /* selected record */
  int                           RecordCount=0; /* total emp records processed */
  int                           Selected=0;     /* total emp records selected */
  int                           SoutSize=0;             /* summary statistics */
  double                        SoutTime=0;             /* summary statistics */
  double                        SoutCount=0;            /* summary statistics */
  double                        CountAvg=0;                 /* eoj statistics */
  double                        TimeAvg=0;                  /* eoj statistics */
  double                        TimeTotal=0;                /* eoj statistics */

    MonFields=(sizeof(FormatMap) / sizeof(FormatMap[0]));  /* number of types */
    for (i=strlen(argv[0]); i>=0; --i)                 /* strip the path info */
      if (argv[0][i] == '/') break;                  /* from the program name */
    strcpy(ProgName, &argv[0][i+1]);                          /* for messages */
    memset(&Table, 0, sizeof(Table));                 /* initialize the table */

    /*----------------------------------------------------*/
    /* Get the command line arguments                     */
    /*----------------------------------------------------*/
    if (cmfopts (argc,                    /* parse the command line arguments */
                 argv,                         /* user argument input strings */
                 MonDataFile,                           /* CMF data file name */
                 TopTitle,                                  /* top title text */
                 FieldList,                         /* selected report fields */
                 IncludeList,                             /* include criteria */
                 ExcludeList))                            /* exclude criteria */
       exit(1);                                         /* exit if any errors */

    /*-----------------------------------------------------*/
    /* Build the temporary row structure.                 -*/
    /*-----------------------------------------------------*/
    if (SummaryRF == TRUE) {
       temp_row.Next = 0;
       temp_row.MajorValue[0] = '\0';
       temp_row.MinorValue[0] = '\0';
       temp_row.ItemCount = 0;
       temp_row.ItemS = (struct SummaryItem_t*) calloc(FieldCount,
                        sizeof(struct SummaryItem_t));
       for(i=0; i < FieldCount; i++)
          temp_row.ItemS[i].Fieldp = FieldList[i].Fieldp;
    }

    if (ReportWidth > LineLth) {
	fprintf(stderr, "[%s]: ReportWidth(%i) is greater than LineLth(%i)\n",
                ProgName, ReportWidth, LineLth);
       exit(1);                                         /* exit if any errors */
    }
    if (strlen(TopTitle) > LineLth-10) {
	fprintf(stderr, "[%s]: TopTitle(%i) is greater than LineLth-10 (%i)\n",
                ProgName, strlen(TopTitle), LineLth-10);
       exit(1);                                         /* exit if any errors */
    }

#ifdef DEBUG
    fprintf(stderr, "[%s]: CMF Data File = \"%s\"\n",
            ProgName, MonDataFile);
    fprintf(stderr, "PageLth=%d, LineLth=%d\n", PageLth, LineLth);
    fprintf(stderr, "Selected fields (%i) = [", FieldCount);
    i2=FieldCount-1;
    for (i=0; i<FieldCount; i++) {             /* for each selected field */
       fprintf(stderr, "%i", FieldList[i].Fieldp->empNum);
       if (i!=i2) fprintf(stderr, ",");
    }
    fprintf(stderr, "]\n");
    if (SummaryRF == TRUE) {
       fprintf(stderr, "Major field: %i\n", Table.Major->empNum);
       if (Table.Minor != NULL)
          fprintf(stderr, "Minor field: %i\n", Table.Minor->empNum);
       else
          fprintf(stderr, "Minor field: none\n");
    }
    if (ExcludeCount > 0)
    {
       fprintf(stderr, "[%s]: Exclude fields (%i) = [",
               ProgName, ExcludeCount);
       for (i=0; i<ExcludeCount; i++)              /* for each selected field */
       {
          fprintf(stderr, "%s", ExcludeList[i].Fieldp->fieldName);
          switch (ExcludeList[i].oper) {
             case GREATERTHAN: fprintf(stderr, ">"); break;
             case LESSTHAN:    fprintf(stderr, "<"); break;
             case EQUALS:      fprintf(stderr, "="); break;
             default:          fprintf(stderr, "?"); break;
          }
          fprintf(stderr, "\"%s\"", ExcludeList[i].text);
          if (i != ExcludeCount-1) fprintf(stderr, ",");
       }
       fprintf(stderr, "]\n");                                /* end the line */
    }
    if (IncludeCount > 0)
    {
       fprintf(stderr, "[%s]: Include fields (%i) = [",
               ProgName, IncludeCount);
       for (i=0; i<IncludeCount; i++)              /* for each selected field */
       {
          fprintf(stderr, "%s", IncludeList[i].Fieldp->fieldName);
          switch (IncludeList[i].oper) {
             case GREATERTHAN: fprintf(stderr, ">"); break;
             case LESSTHAN:    fprintf(stderr, "<"); break;
             case EQUALS:      fprintf(stderr, "="); break;
             default:          fprintf(stderr, "?"); break;
          }
          fprintf(stderr, "\"%s\"", IncludeList[i].text);
          if (i != IncludeCount-1) fprintf(stderr, ",");
       }
       fprintf(stderr, "]\n");                                /* end the line */
    }
#endif

    /*----------------------------------------------------*/
    /* Open the monitor input data file                   */
    /*----------------------------------------------------*/
    MonDataIn = CICS_MFMT_OpenMonitorFile(MonDataFile, &status);
    if (status != CICS_MFMT_STATUS_OK)
    {
	fprintf(stderr, "[%s]: RC(%d) opening monitor file \"%s\"",
                         ProgName, status, MonDataFile);
        switch(status)
        {
           case CICS_MFMT_STATUS_EOF:
               fprintf(stderr, " EOF");
               break;
           case CICS_MFMT_STATUS_SYS_ERROR:
               fprintf(stderr, " SYS_ERROR");
               break;
           case CICS_MFMT_STATUS_FILE_EOF:
               fprintf(stderr, " FILE_EOF");
               break;
           case CICS_MFMT_STATUS_FILE_ERROR:
               fprintf(stderr, " FILE_ERROR");
               break;
           default:
               fprintf(stderr, " UNKNOWN");
               break;
        }

        fprintf(stderr, "\n");
        exit(2);
    }

    /*----------------------------------------------------*/
    /* DEBUG DEBUG DEBUG DEBUG DEBUG DEBUG DEBUG DEBUG    */
    /* This shows the attributes for all field types      */
    /*----------------------------------------------------*/
#ifdef DEBUGTYPES
    for (i=0; i<MonFields; i++)
    {
        fprintf(stderr,"%-12.12s : (%3.3i) Wide %2.2i  Type %8.8X "
                       "\"%15.15s\" ",
                FormatMap[i].fieldName,
                FormatMap[i].empNum,
                FormatMap[i].outSize,
                FormatMap[i].fieldType,
                FormatMap[i].titleTxt);
        switch(FormatMap[i].fieldType)
        {
           case CICS_EMP_TDQ_DETAIL_TYPE_COUNTER:
              fprintf(stderr, "COUNTER\n"); break;
           case CICS_EMP_TDQ_DETAIL_TYPE_TIME_STAMP:
              fprintf(stderr, "TIME_STAMP\n"); break;
           case CICS_EMP_TDQ_DETAIL_TYPE_TIMER:
              fprintf(stderr, "TIMER\n"); break;
           case CICS_EMP_TDQ_DETAIL_TYPE_OCCUPANCY:
              fprintf(stderr, "OCCUPANCY\n"); break;
           case CICS_EMP_TDQ_DETAIL_TYPE_STRING:
              fprintf(stderr, "STRING\n"); break;
           case CICS_EMP_TDQ_DETAIL_TYPE_ABEND_CODE:
              fprintf(stderr, "ABEND_CODE\n"); break;
           case CICS_EMP_TDQ_DETAIL_TYPE_HIGH_WATER:
              fprintf(stderr, "HIGH_WATER\n"); break;
           case CICS_EMP_TDQ_DETAIL_TYPE_OS_COUNT:
              fprintf(stderr, "OS_COUNT\n"); break;
           case CICS_EMP_TDQ_DETAIL_TYPE_OS_CLOCK:
              fprintf(stderr, "OS_CLOCK\n"); break;
           case CICS_EMP_TDQ_DETAIL_TYPE_USER:
              fprintf(stderr, "USER\n"); break;
           default:
              fprintf(stderr, "UNKNOWN?\n"); break;
        }

        ptr = CICS_MFMT_FindField(FormatMap[i].empNum,
              MonDataIn);
        fprintf(stderr, "    (Details)  (%3.3i) Size %2.2i  Type %8.8X "
                        "                  ",
        ptr->FieldID, ptr->FieldSize, ptr->Type);
        switch(ptr->Type)
        {
           case CICS_EMP_TDQ_DETAIL_TYPE_COUNTER:
              fprintf(stderr, "COUNTER\n"); break;
           case CICS_EMP_TDQ_DETAIL_TYPE_TIME_STAMP:
                 fprintf(stderr, "TIME_STAMP\n"); break;
           case CICS_EMP_TDQ_DETAIL_TYPE_TIMER:
              fprintf(stderr, "TIMER\n"); break;
           case CICS_EMP_TDQ_DETAIL_TYPE_OCCUPANCY:
              fprintf(stderr, "OCCUPANCY\n"); break;
           case CICS_EMP_TDQ_DETAIL_TYPE_STRING:
              fprintf(stderr, "STRING\n"); break;
           case CICS_EMP_TDQ_DETAIL_TYPE_ABEND_CODE:
              fprintf(stderr, "ABEND_CODE\n"); break;
           case CICS_EMP_TDQ_DETAIL_TYPE_HIGH_WATER:
              fprintf(stderr, "HIGH_WATER\n"); break;
           case CICS_EMP_TDQ_DETAIL_TYPE_OS_COUNT:
              fprintf(stderr, "OS_COUNT\n"); break;
           case CICS_EMP_TDQ_DETAIL_TYPE_OS_CLOCK:
              fprintf(stderr, "OS_CLOCK\n"); break;
           case CICS_EMP_TDQ_DETAIL_TYPE_USER:
              fprintf(stderr, "USER\n"); break;
           default:
              fprintf(stderr, "UNKNOWN?\n"); break;
        }
    }
    fprintf(stderr, "\n");
    /*----------------------------------------------------*/
    /* DEBUG DEBUG DEBUG DEBUG DEBUG DEBUG DEBUG DEBUG    */
    /*----------------------------------------------------*/
#endif

    /*----------------------------------------------------*/
    /* Format and print the first title line              */
    /*  this will find each selected field in the input   */
    /*  record & create title text based on it's type     */
    /*----------------------------------------------------*/
    if (SummaryRF == FALSE) {               /* standard report detail listing */
       memset(FieldTitle, '\0', 1);                /* start a null title line */
       for (i=0; i<FieldCount; i++)        /* format based on selected fields */
       {
           ptr = CICS_MFMT_FindField(FieldList[i].Fieldp->empNum,
                 MonDataIn);
           MonTitle(ptr, FieldList[i].Fieldp, field);           /* title text */
           strcat(FieldTitle, " ");             /* add a blank between fields */
           strcat(FieldTitle, field);             /* add it to the title line */
       }
    } else {                                        /* summary report listing */
       strcpy(FieldTitle, " ");                       /* start with one blank */
       strcat(FieldTitle, Table.Major->titleTxt);    /* title of major column */
       if (Table.Minor != NULL) {                    /* title of minor column */
          strcat(FieldTitle, " ");              /* add a blank between fields */
          strcat(FieldTitle, Table.Minor->titleTxt);
       }
       strcat(FieldTitle, "  Count");           /* count of items in this row */
       for (i=0; i<FieldCount; i++)        /* format based on selected fields */
       {                                   /* exclude major and minor columns */
           if ((Table.Major->empNum != temp_row.ItemS[i].Fieldp->empNum) &&
               ((Table.Minor == NULL) ? 1 :
                (Table.Minor->empNum != temp_row.ItemS[i].Fieldp->empNum))) {
              ptr = CICS_MFMT_FindField(FieldList[i].Fieldp->empNum,
                    MonDataIn);
              MonTitle(ptr, temp_row.ItemS[i].Fieldp, field);   /* title text */
              strcat(FieldTitle, " ");          /* add a blank between fields */
              strcat(FieldTitle, field);          /* add it to the title line */
           }
       }
    }
    if (TopTitle[0] != '\0')
       printf(" Page %03d %s\n", PageNo, TopTitle);        /* user title text */
    if (TitleLines > 0)
       if (PrtDetail)                                /* summary report only ? */
          printf("%s\n", FieldTitle);           /* display the field headings */

    /*----------------------------------------------------*/
    /* Process the monitor data and format report details */
    /*----------------------------------------------------*/
    while (! atEOF)
    {
        if (SummaryRF == FALSE) {
           if (LineNo > PageLth)               /* is it time for a new page ? */
           {
              PageNo++;                   /* incriment the current page count */
              if (TopTitle[0] != '\0' && PrtDetail)
                 printf(" Page %03d %s\n", PageNo, TopTitle);/* usr title txt */
              if (TitleLines > 0 && PrtDetail)
                 printf("%s\n", FieldTitle);                /* field headings */
              LineNo=1;                             /* start a new line count */
           }
        }

        /*---------------------------------------------*/
        /* Process record selection options ...        */
        /*---------------------------------------------*/
        Select = TRUE;                                             /* default */
        if (IncludeCount > 0)        /* looking for specific included records */
        {
           Select = FALSE;                                         /* default */
           for (i=0; i<IncludeCount; i++)          /* for each included field */
           {
               ptr = CICS_MFMT_FindField(IncludeList[i].Fieldp->empNum,
                     MonDataIn);
               if (MonCompare(ptr, IncludeList[i])) {      /* compare a field */
                  Select = TRUE;                            /* we got a match */
                  break;                         /* time to quit for includes */
               }
           }
        }

        if (Select == TRUE && ExcludeCount > 0)  /* specific excluded records */
        {
           Select = TRUE;                                          /* default */
           for (i=0; i<ExcludeCount; i++)          /* for each excluded field */
           {
               ptr = CICS_MFMT_FindField(ExcludeList[i].Fieldp->empNum,
                     MonDataIn);
               if (MonCompare(ptr, ExcludeList[i])) {      /* compare a field */
                  Select = FALSE;                           /* we got a match */
                  break;                         /* time to quit for excludes */
               }
           }
        }

        /*---------------------------------------------*/
        /* Format & accumulate all requested fields    */
        /*---------------------------------------------*/
        if (Select == TRUE)                /* is this a selected CMF record ? */
        {
           if (SummaryRF == TRUE) {                         /* Summary Only ? */
              temp_row.Next = 0;                       /* initialize temp row */
              temp_row.MajorValue[0] = '\0';
              temp_row.MinorValue[0] = '\0';
              temp_row.ItemCount = 0;
              for(i=0; i<FieldCount; i++) {   /* don't change Fieldp pointers */
                 temp_row.ItemS[i].RowCount = 0;                /* initialize */
                 temp_row.ItemS[i].RowTime = 0;                 /* initialize */
              }
           } else {                                      /* Full Detail Lines */
              memset(DetailLine, '\0', 1);               /* set a null string */
           }

           /*---------------------------------------------*/
           /* Capture the data for each requested CMF EMP */
           /*---------------------------------------------*/
           for (i=0; i<FieldCount; i++)
           {
               ptr = CICS_MFMT_FindField(FieldList[i].Fieldp->empNum,
                     MonDataIn);                       /* find the next field */
               MonExpand(ptr, FieldList[i].Fieldp,
                         field, &temp_row);                      /* expand it */
               if (SummaryRF == FALSE) {
                  strcat(DetailLine, " ");      /* add a space between fields */
                  strcat(DetailLine, field);  /* add field to the detail line */
               }
           }

           /*--------------------------------------------------------*/
           /* Either print the current line or accumulate its totals */
           /*--------------------------------------------------------*/
           if (SummaryRF == TRUE) {
              if (insertRow(&temp_row) != 0) {     /* accumulate summary data */
                 fprintf(stderr,
                         "Could not build output table for accumulation\n");
                 exit(1);
              }
           } else {
              if (PrtDetail)                         /* summary report only ? */
                 printf("%s\n", DetailLine);           /* print a detail line */
              LineNo++;                                /* count lines printed */
           }
           Selected++;                          /* count the selected records */
        } /* end if (Select0 */

        /*-----------------------------------------------------*/
        /* Get the next CMF record and check for EOF or errors */
        /*-----------------------------------------------------*/
        status=CICS_MFMT_ReadNext(MonDataIn);     /* get the next task record */
        if (status != CICS_MFMT_STATUS_OK)
        {
           atEOF=TRUE;                                /* signal end of input */
           switch(status)                       /* report any abnormal error */
           {
              case CICS_MFMT_STATUS_EOF:                  /* this is expected */
                 break;
              case CICS_MFMT_STATUS_SYS_ERROR:
	         fprintf(stderr, "[%s]: "
                        "CICS_MFMT_ReadNext(%i) RC=SYS_ERROR\n",
                        ProgName, status);
                 exit(1);
                 break;
              case CICS_MFMT_STATUS_FILE_EOF:
	         fprintf(stderr, "[%s]: "
                        "CICS_MFMT_ReadNext(%i) RC=UNEXPECTED_EOF\n",
                        ProgName, status);
                 exit(1);
                 break;
              case CICS_MFMT_STATUS_FILE_ERROR:
	         fprintf(stderr, "[%s]: "
                        "CICS_MFMT_ReadNext(%i) RC=FILE_ERROR\n",
                        ProgName, status);
                 exit(1);
                 break;
              default:
	         fprintf(stderr, "[%s]: "
                        "CICS_MFMT_ReadNext(%i) RC=UNKNOWN?\n",
                        ProgName, status);
                 exit(1);
                 break;
           } /* end switch */
	} /* end if-status */
        RecordCount++;                             /* Count the input records */
    } /* end while (! atEOF) */

    /*------------------------------------------------------------------------*/
    /* The summary data and final totals have been accumulated.  It is time   */
    /* to print the summary tables that have been processed if this was a     */
    /* summary run.  In any case, the final totals will be printed next.      */
    /*------------------------------------------------------------------------*/
    if (SummaryRF == TRUE) {
       free(temp_row.ItemS);               /* this memory is no longer needed */
       while (Table.Head != NULL) {  /* process each row in the summary table */
          if (LineNo > PageLth)                /* is it time for a new page ? */
          {
             /*------------------------------------------*/
             /* Print new titles at the end of each page */
             /*------------------------------------------*/
             PageNo++;                    /* incriment the current page count */
             if (TopTitle[0] != '\0' && PrtDetail)
                printf(" Page %03d %s\n", PageNo, TopTitle); /* usr title txt */
             if (TitleLines > 0 && PrtDetail)
                printf("%s\n", FieldTitle);                 /* field headings */
             LineNo=1;                              /* start a new line count */
          }
          strcpy(DetailLine, " ");                    /* start with one blank */
          /*-----------------------------------------*/
          /* Display the major & minor fields        */
          /*-----------------------------------------*/
          strcat(DetailLine, Table.Head->MajorValue);     /* show major field */
          if (Table.Minor != NULL) {
             strcat(DetailLine, " ");                 /* space between fields */
             strcat(DetailLine, Table.Head->MinorValue);        /* show minor */
          }
          /*-----------------------------------------*/
          /* Display the row item count              */
          /*-----------------------------------------*/
          sprintf(field, " %6.0f", Table.Head->ItemCount); /* show item count */
          strcat(DetailLine, field);               /* right after major/minor */
          /*-----------------------------------------*/
          /* Display each column in the summary line */
          /*-----------------------------------------*/
          for(i=0; i<FieldCount; i++) {
             if ((Table.Major->empNum != Table.Head->ItemS[i].Fieldp->empNum) &&
                 ((Table.Minor == NULL) ? 1 :
                  (Table.Minor->empNum != Table.Head->ItemS[i].Fieldp->empNum)))
             {
                strcat(DetailLine, " ");              /* space between fields */
                SoutSize=Table.Head->ItemS[i].Fieldp->outSize;  /* field size */
                SoutCount=Table.Head->ItemS[i].RowCount;        /* item count */
                SoutCount = SoutCount / Table.Head->ItemCount;     /* average */
                switch(Table.Head->ItemS[i].Fieldp->fieldType)
                {
                   case CICS_EMP_TDQ_DETAIL_TYPE_OS_COUNT:
                   case CICS_EMP_TDQ_DETAIL_TYPE_COUNTER:
                      sprintf(field, "%*.2f", SoutSize, SoutCount);
                      strcat(DetailLine, field);
                      break;
                   case CICS_EMP_TDQ_DETAIL_TYPE_TIMER:
                   case CICS_EMP_TDQ_DETAIL_TYPE_OS_CLOCK:
                      SoutTime=Table.Head->ItemS[i].RowTime / 1000;
                      SoutTime = SoutTime / Table.Head->ItemCount;
                      sprintf(field, "%*.3f", SoutSize, SoutTime);
                      strcat(DetailLine, field);
                      if ((PrtCount==TRUE) &&
                          (Table.Head->ItemS[i].Fieldp->fieldType
                           == CICS_EMP_TDQ_DETAIL_TYPE_TIMER)) {
                         sprintf(field, " %6.1f", SoutCount);
                         strcat(DetailLine, field);
                      }
                      break;
                   default:
                      break;
                }
             }
          }

          /*-----------------------------------------*/
          /* End the line & incriment the line count */
          /*-----------------------------------------*/
          printf("%s\n", DetailLine);
          LineNo++;

          /*-----------------------------------------*/
          /* Free the row & adjust the pointers      */
          /*-----------------------------------------*/
          free(Table.Head->ItemS);
          tmpPtr = Table.Head->Next;
          free(Table.Head);
          Table.Head = tmpPtr;
       }
    } /* end if (SummaryRF) */

    /*---------------------------------------------*/
    /* End of Job: Display some summary statistics */
    /*---------------------------------------------*/
    fprintf(stderr, "[%s]:  Selected fields(%i) "
           "Selected Records = %i of %i\n",
            ProgName, FieldCount, Selected, RecordCount);
    if (SummaryF)                             /* unless summary is suppressed */
       printf("\n\n[%s]:  Selected EMPs(%i) Selected Records = %i of %i\n",
              ProgName, FieldCount, Selected, RecordCount);

    if ((SummaryF) && (Selected !=0))         /* unless summary is suppressed */
    {
       printf("         name emp  type    average         total (avg-count)\n");
       for (i=0; i<FieldCount; i++)                /* for each selected field */
       {
          ReportItem=FieldList[i].Fieldp;         /* report field description */

          switch(ReportItem->fieldType)
          {
            case CICS_EMP_TDQ_DETAIL_TYPE_OS_COUNT:
            case CICS_EMP_TDQ_DETAIL_TYPE_COUNTER:
               if (ReportItem->empNum == CICS_EMP_FID_TCASEQ_NO ||
                   ReportItem->empNum == CICS_EMP_FID_TASK_FLAGS)
                   break;
               CountAvg = ReportItem->ValueCtr / Selected;
               printf(" %12s %03i Count %10.3f  %12.0f\n",
                      ReportItem->fieldName, ReportItem->empNum,
                      CountAvg, ReportItem->ValueCtr);
               break;

            case CICS_EMP_TDQ_DETAIL_TYPE_TIMER:
               CountAvg = ReportItem->ValueCtr / Selected;
               TimeTotal = ReportItem->ValueTime;
               TimeAvg = TimeTotal / Selected;
               printf(" %12s %03i Timer %10.3f  %12.3f %9.3f\n",
                      ReportItem->fieldName, ReportItem->empNum,
                      TimeAvg/1000, TimeTotal/1000, CountAvg);
               break;

            case CICS_EMP_TDQ_DETAIL_TYPE_OS_CLOCK:
               TimeTotal = ReportItem->ValueTime;
               TimeAvg = TimeTotal / Selected;
               printf(" %12s %03i Clock %10.3f  %12.3f\n",
                      ReportItem->fieldName, ReportItem->empNum,
                      TimeAvg/1000, TimeTotal/1000);
               break;

            case CICS_EMP_TDQ_DETAIL_TYPE_USER:
                 switch(ReportItem->usrType)       /* based on it's user type */
                 {
                    case Clk1T:
                    case Clk2T:
                       TimeTotal = ReportItem->ValueTime;
                       TimeAvg = TimeTotal / Selected;
                       printf(" %12s %03i Clock %10.3f  %12.3f\n",
                              ReportItem->fieldName, ReportItem->empNum,
                              TimeAvg/1000, TimeTotal/1000);
                       break;
                    case Clk1Ct:
                    case Clk2Ct:
                    case Ctr1:
                    case Ctr2:
                    case Ctr3:
                       CountAvg = ReportItem->ValueCtr / Selected;
                       printf(" %12s %03i Count %10.3f  %12.0f\n",
                              ReportItem->fieldName, ReportItem->empNum,
                              CountAvg, ReportItem->ValueCtr);
                       break;
                    default:
                       break;
                 }
               break;

            default:
               break;
          }

       } /* end for (FieldCount) */
    } /* end if (SummaryF) */

    exit(0);
} /* end main() */

